@@ -1,62 +1,45 @@
 #!/bin/env python
 
 import os
-import getpass
 import functools
 import codecs
+import warnings
+import contextlib
+import getpass
 
 import requests
 import msgpack
-import msgpack_numpy as mn
 import numpy as np
+import urllib3.exceptions
+from requests.auth import HTTPBasicAuth
 
 from hdfstream.exceptions import HDFStreamRequestError
+from hdfstream.decoding import decode_response
 
 
-# Allow disabling SSL certificate verification and associated warnings (useful for testing)
-verify_cert = True
-def disable_verify_cert():
-    global verify_cert
-    import urllib3
-    urllib3.disable_warnings()
-    verify_cert = False
-
-
-def decode_hook(data):
+_verify_cert = True
+def verify_cert(enable):
     """
-    Converts dicts decoded from the msgpack stream into numpy ndarrays. Called
-    by msgpack.unpack().
+    Disable SSL certificate validation. Should only be used for testing.
 
-    Dicts with nd=True contain a binary buffer which should be wrapped with a
-    numpy array with type and shape given by the metadata in the dict. We call
-    msgpack-numpy's decode() function to do this.
-
-    Dicts with vlen=True contain flattened lists of variable size
-    elements and are translated into numpy object arrays.
+    :param enable: whether to validate the server's certificate
+    :type enable: bool
     """
+    global _verify_cert
+    _verify_cert = enable
 
-    # If this is a serialized ndarray, use msgpack-numpy to decode it.
-    # Need to translate any string keys to bytes first.
-    if isinstance(data, dict) and "nd" in data:
-        result = {}
-        for name in data:
-            result[name.encode(encoding="ascii")] = data[name]
-        data = mn.decode(result).copy() # copy to ensure writable
 
-    # Then check for any vlen data: in that case we have a flattened list
-    # which needs to be converted into an object array of the right shape.
-    if isinstance(data, dict) and "vlen" in data:
-        # Get the shape of the array
-        shape = [int(i) for i in data["shape"]]
-        if len(shape) == 0:
-            # For scalars, just return the value
-            data = data["data"][0]
-        else:
-            # Otherwise we make an object array
-            arr = np.empty(len(data["data"]), object)
-            arr[:] = data["data"]
-            data = arr.reshape(shape)
-    return data
+# Context manager to suppress certificate warnings if _verify=False
+@contextlib.contextmanager
+def _maybe_suppress_cert_warnings():
+    if _verify_cert:
+        # Certificate checking enabled, so do nothing
+        yield
+    else:
+        # Certificate checking disabled, so silence warnings
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', urllib3.exceptions.InsecureRequestWarning)
+            yield
 
 
 def raise_for_status(response):
@@ -96,12 +79,18 @@ class Connection:
         # Remove any trailing slashes from the server name
         self.server = server.rstrip("/")
 
+        # If a username is specified with no password, prompt for the password
+        if user is not None and password is None:
+            password = getpass.getpass()
+
         # Set up a session with the username and password
         self.session = requests.Session()
-        self.session.auth = (user, password)
+        if user is not None:
+            self.session.auth = HTTPBasicAuth(user, password)
 
         # Test by fetching a root directory listing
-        response = self.session.get(self.server+"/msgpack/", verify=verify_cert)
+        with _maybe_suppress_cert_warnings():
+            response = self.session.get(self.server+"/msgpack/", verify=_verify_cert)
         raise_for_status(response)
 
     @staticmethod
@@ -119,52 +108,75 @@ class Connection:
             Connection._cache[connection_id] = Connection(server, user, password)
         return Connection._cache[connection_id]
 
-    def get_and_unpack(self, url, params=None):
+    def get_and_unpack(self, url, params=None, desc=None):
         """
         Make a request and unpack the response
         """
-        with self.session.get(url, params=params, stream=True, verify=verify_cert) as response:
-            raise_for_status(response)
-            response.raw.read = functools.partial(response.raw.read, decode_content=True)
-            data = msgpack.unpack(response.raw, object_hook=decode_hook)
+        with _maybe_suppress_cert_warnings():
+            with self.session.get(url, params=params, stream=True, verify=_verify_cert) as response:
+                raise_for_status(response)
+                data = decode_response(response, desc)
         return data
 
     def request_path(self, path):
         """
         Request the msgpack representation of a file or directory from the server
         """
+        path = path.lstrip("/")
         url = f"{self.server}/msgpack/{path}"
-        return self.get_and_unpack(url)
+        return self.get_and_unpack(url, desc=f"Path: {path}")
 
     def request_object(self, path, name, data_size_limit, max_depth):
         """
         Request the msgpack representation of a HDF5 object from the server
         """
+        path = path.lstrip("/")
         params = {
             "object" : name,
             "data_size_limit" : data_size_limit,
             "max_depth" : max_depth
         }
         url = f"{self.server}/msgpack/{path}"
-        return self.get_and_unpack(url, params)
+        return self.get_and_unpack(url, params, desc=f"Object: {name}")
 
     def request_slice(self, path, name, slice_string):
         """
-        Request a dataset slice.
+        Request a dataset slice. Returns a new np.ndarray.
         """
+        path = path.lstrip("/")
         params = {
             "object" : name,
             "slice"  : slice_string,
         }
         url = f"{self.server}/msgpack/{path}"
-        return self.get_and_unpack(url, params)
+        return self.get_and_unpack(url, params, desc=f"Slice: {name}")
+
+    def request_slice_into(self, path, name, slice_string, destination):
+        """
+        Request a dataset slice and read it into the supplied buffer.
+
+        Will only work for fixed length data types.
+        """
+        path = path.lstrip("/")
+        params = {
+            "object" : name,
+            "slice"  : slice_string,
+        }
+        url = f"{self.server}/msgpack/{path}"
+        with _maybe_suppress_cert_warnings():
+            with self.session.get(url, params=params, stream=True, verify=_verify_cert) as response:
+                raise_for_status(response)
+                decode_response(response, desc=f"Slice: {name}", destination=destination)
 
     def open_file(self, path, mode='r'):
         """
         Open the file at the specified virtual path
         """
+        path = path.lstrip("/")
         url = f"{self.server}/download/{path}"
-        response = self.session.get(url, stream=True, verify=verify_cert)
+
+        with _maybe_suppress_cert_warnings():
+            response = self.session.get(url, stream=True, verify=_verify_cert)
         raise_for_status(response)
         response.raw.read = functools.partial(response.raw.read, decode_content=True)
         if mode == 'rb':
