@@ -4,25 +4,34 @@ import numpy as np
 import collections.abc
 
 
-class RemoteDatasetSlicer:
-
-    def __init__(self, dataset):
-        self.dataset = dataset
-
-    def __getitem__(self, key):
-        return key
-
-
 class RemoteDataset:
     """
-    Object representing a HDF5 dataset in the remote file
+    This class represents a HDF5 dataset in a file on the server. To open a
+    dataset, index the parent RemoteGroup or RemoteFile object. The class
+    constructor documented here is used to implement lazy loading of HDF5
+    metadata and should not usually be called directly.
 
-    Parameters:
+    Indexing a RemoteDataset with numpy style slicing yields a numpy array
+    with the dataset contents. Slices must be contiguous ranges. Indexing
+    with an array is not supported.
 
-    connection: Connection object to use to send requests
-    file_path: path to the file containing the HDF5 dataset
-    name: name of the HDF5 dataset to open
-    data: msgpack encoded dataset description
+    :param connection: connection object which stores http session information
+    :type connection: hdfstream.connection.Connection
+    :param file_path: virtual path of the file containing the dataset
+    :type file_path: str
+    :param name: name of the HDF5 dataset
+    :type name: str
+    :param data: decoded msgpack data describing the dataset, defaults to None
+    :type data: dict, optional
+    :param parent: parent HDF5 group, defaults to None
+    :type parent: hdfstream.RemoteGroup, optional
+
+    :ivar attrs: dict of HDF5 attribute values of the form {name : np.ndarray}
+    :vartype attrs: dict
+    :ivar dtype: data type for this dataset
+    :vartype dtype: np.dtype
+    :ivar shape: shape of this dataset
+    :vartype shape: tuple of integers
     """
     def __init__(self, connection, file_path, name, data, parent):
 
@@ -52,7 +61,7 @@ class RemoteDataset:
             if hasattr(arr, "shape") and len(arr.shape) == 0:
                 self.attrs[name] = arr[()]
 
-    def make_slice_string(self, key):
+    def _make_slice_string(self, key):
         """
         Given a key suitable for indexing an ndarray, generate a slice
         specifier string for the web API.
@@ -114,7 +123,7 @@ class RemoteDataset:
             key = (key,)
 
         # Convert the key to a slice string
-        slice_string, dim_mask = self.make_slice_string(key)
+        slice_string, dim_mask = self._make_slice_string(key)
 
         if self.data is None:
             # Dataset is not in memory, so request it from the server
@@ -136,15 +145,42 @@ class RemoteDataset:
 
     def read_direct(self, array, source_sel=None, dest_sel=None):
         """
-        Mimic h5py's Dataset.read_direct() method.
+        Read data directly into a destination buffer. This can
+        save time by preventing unneccessary copying of the data but
+        only works for fixed length types (e.g. integer or floating
+        point data).
 
-        For compatibility only. There's no performance benefit here.
+        Copies the data if the destination array does not have the same data
+        type as the dataset.
+
+        :param array: output array which will receive the data
+        :type array: np.ndarray
+        :param source_sel: selection in the source dataset as a numpy slice, defaults to None
+        :type source_sel: slice or list of slices, optional
+        :param dest_sel: selection in the output array as a numpy slice, defaults to None
+        :type dest_sel: slice or list of slices, optional
         """
         if source_sel is None:
             source_sel = Ellipsis
         if dest_sel is None:
             dest_sel = Ellipsis
-        array[dest_sel] = self[source_sel]
+        slice_string, _ = self._make_slice_string(source_sel)
+
+        # Get a view of the destination selection, making sure we do not make a copy
+        dest_view = array[dest_sel]
+        if not dest_view.flags['C_CONTIGUOUS']:
+            raise RuntimeError("Destination for read_direct() must be C contiguous")
+        if not np.shares_memory(dest_view, array):
+            raise RuntimeError("Unable to read directly into specified selection")
+
+        if array.dtype == self.dtype:
+            # The data types match, so we can download directly into the destination buffer
+            self.connection.request_slice_into(self.file_path, self.name, slice_string, dest_view)
+        else:
+            # The data types are different, so we have to make a copy and let numpy convert the values
+            if not np.can_cast(self.dtype, array.dtype, casting='safe'):
+                raise RuntimeError("Cannot safely cast {self.dtype} to {array.dtype}")
+            dest_view[...] = self.connection.request_slice(self.file_path, self.name, slice_string)
 
     def __len__(self):
         if len(self.shape) >= 1:
@@ -154,36 +190,45 @@ class RemoteDataset:
 
     def close(self):
         """
-        There's nothing to close, but some code might expect this to exist
+        Close the group. Only included for compatibility (there's nothing to close.)
         """
         pass
 
-    def slicer(self):
-        return RemoteDatasetSlicer(self)
-
-    def request_slices(self, keys):
+    def request_slices(self, keys, dest=None):
         """
-        Request a series of dataset slices from the server
+        Request a series of dataset slices from the server and return a single
+        array with the slices concatenated along the first dimension. Slices
+        may only differ in the first dimension, must be in ascending order of
+        starting index in the first dimension, and must not overlap. Example
+        usage::
 
-        Returns a single array with data concatenated along the first
-        axis. Intended usage is something like this:
+          slices = []
+          slices.append(np.s_[0:10,:])
+          slices.append(np.s_[100:110,:])
+          result = dataset.request_slices(slices)
 
-        slices = []
-        ds = dataset.slicer()
-        slices.append(ds[0:10,:])
-        slices.append(ds[100:110,:])
-        result = dataset.request_slices(slices)
+        If the optional dest parameter is used the result is written to dest.
+        Otherwise a new np.ndarray is returned.
+
+        :param keys: list of slices to read
+        :type keys: list of slices
+        :param dest: destination buffer to write to, defaults to None
+        :type dest: np.ndarray, optional
         """
         # Construct the slice specifier string
         slices = []
         for key in keys:
-            slice_string, dim_mask = self.make_slice_string(key)
+            slice_string, dim_mask = self._make_slice_string(key)
             slices.append(slice_string)
         slice_string = ";".join(slices)
 
-        # Request the data
-        data = self.connection.request_slice(self.file_path, self.name, slice_string)
+        if dest is None:
+            # Make the request and return a new array
+            data = self.connection.request_slice(self.file_path, self.name, slice_string)
+            # Remove dimensions where the index was a scalar
+            result_dims = np.asarray(data.shape, dtype=int)[dim_mask]
+            return data.reshape(result_dims)
+        else:
+            # Download the data into the supplied destination array's buffer
+            self.connection.request_slice_into(self.file_path, self.name, slice_string, dest)
 
-        # Remove dimensions where the index was a scalar
-        result_dims = np.asarray(data.shape, dtype=int)[dim_mask]
-        return data.reshape(result_dims)
